/**
 * BLOCK: plusbf
 *
 * Registering a basic block with Gutenberg.
 * Simple block, renders and saves the same content without any interactivity.
 */

import './editor.scss';
import './style.scss';
import HelpTooltip from './help';

const { __ } = wp.i18n;
const { assign } = lodash;
const { Fragment } = wp.element;
const { addFilter } = wp.hooks;
const {
	PanelBody,
	TextareaControl,
	TextControl,
} = wp.components;

const { InspectorControls } = wp.editor;

const { createHigherOrderComponent } = wp.compose;

const {
	select,
	subscribe,
} = wp.data;

const { parse } = wp.blocks;

const isValidBlockType = ( name ) => {
	let re = new RegExp('(^core|^core-embed)\/', 'is');
	let matches = re.exec(name);
	const inValidBlockTypes = [
		'core/html',
		'core/block',
		'core/nextpage',
		'core/shortcode',
		'core/freeform',
		'core/more',
	];
	return matches !== null && !inValidBlockTypes.includes( name );
};

/**
 * 空白で分けた文字列の配列を返す
 * @param str
 * @returns {*}
 */
const spilitSpace = (str) => {
	let strings =   str.split(/\s/);
	return strings.filter(function(string) {
		return string !== '';
	});
};


/**
 * フォーム追加
 */
export const addBlockControl = createHigherOrderComponent( ( BlockEdit ) => {
	return ( props, state ) => {
		// isValidBlockType で指定したブロックが選択されたら表示
		if (  props.isSelected && isValidBlockType( props.name ) ) {

			if (typeof props.attributes.plusbfClassName === 'undefined') {
				props.setAttributes( { plusbfClassName: '' } );
			}
			let plusbfclassName = props.attributes.plusbfClassName;

			if (typeof props.attributes.plusbfStyle === 'undefined') {
				props.setAttributes( { plusbfStyle: '' } );
			}
			let plusbfStyle = props.attributes.plusbfStyle;

			return (
				<Fragment>
					<BlockEdit { ...props } />
					<InspectorControls>
						<PanelBody title="スタイルPLUS" initialOpen={ true } className="plusbf-controle-panel">
							<HelpTooltip
								contents="選択したブロックにクラス名（半⾓）を追加できます。こちらは⾼度な設定と同じです。先頭に数字は使⽤しないでください。"
							/>
							<TextControl
								label="クラス名"
								value={ plusbfclassName }
								onChange={ ( plusbfclassNameVal ) => {
									let plusbfClassNames = spilitSpace( plusbfclassNameVal );
									let newClassName = plusbfClassNames.join(' ');
									if (props.attributes.className) {
										//追加CSSクラス
										let classNameVal = props.attributes.className;
										//前のclass名を一度削除
										let statPrevClassName = state.preplusbfClassName;
										if (typeof statPrevClassName === 'undefined') {
											statPrevClassName = '';
										}

										let prevClassName = spilitSpace(statPrevClassName);
										let classNames = spilitSpace(classNameVal);

										//plusbfのclassにないものだけ抜き出す
										classNames.forEach((className) => {
											if ( !plusbfClassNames.includes(className) && !prevClassName.includes(className)) {
												newClassName = newClassName + ' ' + className;
											}
										});
									}
									state.preplusbfClassName = plusbfclassNameVal;

									props.setAttributes({
										plusbfClassName: plusbfclassNameVal,
										className: newClassName
									});

								}}
							/>
							<HelpTooltip
								contents="選択したブロックにCSSを追加できます。"
							/>
							<TextareaControl
								label="スタイル"
								id="plusbf-style-code"
								className="plusbf-style-code"
								value={ plusbfStyle }
								onChange={ ( plusbfStyleVal ) => {
									props.setAttributes({
										plusbfStyle: plusbfStyleVal
									});

								}}
							/>
							<ul className="plusbf-links">
								<li><a href="https://plusbf.com/" target="_blank">公式でデザインを探す</a></li>
							</ul>
						</PanelBody>
					</InspectorControls>
				</Fragment>
			);
		}
		return <BlockEdit { ...props } />;
	};
}, 'addMyCustomBlockControls' );

addFilter( 'editor.BlockEdit', 'plusbf/block-control', addBlockControl );

export function addAttribute( props, name ) {
	if ( isValidBlockType( name ) ) {
		props.attributes = assign( props.attributes, {
			plusbfClassName: {
				type: 'string',
				default: '',
			},
			plusbfStyle: {
				type: 'string',
				default: '',
			},
		} );
	}
	return props;
}
addFilter( 'blocks.registerBlockType', 'plusbf/add-attr', addAttribute );

const addEditStyle = ( style ) => {
	let element = document.getElementById( 'plusbf-edit-style' );

	if ( element === null) {
		element = document.createElement( 'style' );
		element.setAttribute( 'type', 'text/css' );
		element.setAttribute( 'id', 'plusbf-edit-style' );
		document.getElementsByTagName( 'head' )[0].appendChild( element );
	}
	return element.textContent = style;
};

let styles = [];
const buildStyle = ( blocks, reusableBlocks ) => {
	blocks.forEach( block => {
		if ( typeof block.attributes.plusbfStyle !== 'undefined') {
			styles.push(block.attributes.plusbfStyle);
		}
		if ( 'core/block' === block.name && typeof block.attributes.ref !== 'undefined' &&  reusableBlocks !== null ) {
			//再利用ブロックのCSSを取得
			let reBlocks = reusableBlocks.find( reBlock => reBlock.id === block.attributes.ref );
			if ( reBlocks ) {
				buildStyle( parse( reBlocks.content.raw ), reusableBlocks );
			}
		}
		if (  undefined !== block.innerBlocks && 0 < ( block.innerBlocks ).length ) {
			buildStyle( block.innerBlocks, reusableBlocks );
		}
	});
	return styles.join("\n");
};



subscribe( () => {
	styles = [];
	const blocks = select('core/block-editor').getBlocks();
	const reusableBlocks = select( 'core' ).getEntityRecords( 'postType', 'wp_block' );
	const style = buildStyle( blocks,  reusableBlocks);
	addEditStyle( style );
});
